@@ -8,10 +8,44 @@ Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
                    All rights reserved
 
 Created: Fri Mar 17 17:09:37 1995 ylo
-Last modified: Wed Jul 12 01:24:08 1995 ylo
+
+Generic header file for ssh.
 
 */
 
+/*
+ * $Id: ssh.h,v 1.9 1995/08/31 09:23:42 ylo Exp $
+ * $Log: ssh.h,v $
+ * Revision 1.9  1995/08/31  09:23:42  ylo
+ * 	Added support for ETCDIR.
+ *
+ * Revision 1.8  1995/08/29  22:33:52  ylo
+ * 	Added get_remote_ipaddr, get_permanent_fd.
+ * 	Deleted SSH_NUM_DUPS.
+ *
+ * Revision 1.7  1995/08/21  23:28:57  ylo
+ * 	Added SERVER_CONFIG_FILE.
+ * 	Added syslog facility definitions.
+ *
+ * Revision 1.6  1995/08/18  22:57:06  ylo
+ * 	Now uses 3DES as the authfile cipher if IDEA is not available.
+ *
+ * Revision 1.5  1995/07/27  00:41:04  ylo
+ * 	Added GlobalKnownHostsFile and UserKnownHostsFile.
+ *
+ * Revision 1.4  1995/07/26  23:28:56  ylo
+ * 	Changed PROTOCOL_MINOR from 0 to 1.
+ *
+ * Revision 1.3  1995/07/16  01:02:26  ylo
+ * 	Removed host argument from record_logout.
+ *
+ * Revision 1.2  1995/07/13  01:40:22  ylo
+ * 	Removed "Last modified" header.
+ * 	Added cvs log.
+ *
+ * $Endlog$
+ */
+
 #ifndef SSH_H
 #define SSH_H
 
@@ -26,7 +60,7 @@ Last modified: Wed Jul 12 01:24:08 1995 ylo
 
 /* Minor protocol version.  Different version indicates minor incompatibility
    that does not prevent interoperation. */
-#define PROTOCOL_MINOR		0
+#define PROTOCOL_MINOR		1
 
 /* Name for the service.  The port named by this service overrides the default
    port if present. */
@@ -34,20 +68,21 @@ Last modified: Wed Jul 12 01:24:08 1995 ylo
 
 /* System-wide file containing host keys of known hosts.  This file should be
    world-readable. */
-#define SSH_SYSTEM_HOSTFILE	"/etc/ssh_known_hosts"
+#define SSH_SYSTEM_HOSTFILE	ETCDIR "/ssh_known_hosts"
 
-/*  HOST_KEY_FILE		/etc/ssh_host_key
+/*  HOST_KEY_FILE		/etc/ssh_host_key,
+    SERVER_CONFIG_FILE		/etc/sshd_config,
 and HOST_CONFIG_FILE		/etc/ssh_config
-are both defined in Makefile.in.  Of these, ssh_host_key should be readable
+are all defined in Makefile.in.  Of these, ssh_host_key should be readable
 only by root, whereas ssh_config should be world-readable. */
 
 /* Random seed file for the daemon.  This file should be readable only by 
    root. */
-#define SSH_DAEMON_SEED_FILE	"/etc/ssh_random_seed"
+#define SSH_DAEMON_SEED_FILE	ETCDIR "/ssh_random_seed"
 
 /* The process id of the daemon listening for connections is saved
    here to make it easier to kill the correct daemon when necessary. */
-#define SSH_DAEMON_PID_FILE	"/etc/sshd_pid"
+#define SSH_DAEMON_PID_FILE	ETCDIR "/sshd_pid"
 
 /* The directory in user\'s home directory in which the files reside.
    The directory should be world-readable (though not all files are). */
@@ -56,7 +91,7 @@ only by root, whereas ssh_config should be world-readable. */
 /* Per-user file containing host keys of known hosts.  This file need
    not be readable by anyone except the user him/herself, though this does
    not contain anything particularly secret. */
-#define SSH_USER_HOSTFILE	".ssh/known_hosts"
+#define SSH_USER_HOSTFILE	"~/.ssh/known_hosts"
 
 /* Name of the file containing client-side random seed.  This file should
    only be readable by the user him/herself. */
@@ -107,16 +142,15 @@ only by root, whereas ssh_config should be world-readable. */
 #define SSH_AUTHSOCKET_ENV_NAME	"SSH_AUTHENTICATION_SOCKET"
 
 /* Cipher used for encrypting authentication files. */
+#ifdef WITHOUT_IDEA
+#define SSH_AUTHFILE_CIPHER	SSH_CIPHER_3DES
+#else
 #define SSH_AUTHFILE_CIPHER	SSH_CIPHER_IDEA
+#endif
 
 /* Default port number. */
 #define SSH_DEFAULT_PORT	22
 
-/* Number of file descriptors to dup before creating authentication fd.
-   These are used to move the authentication fd rather high, because some
-   shells arbitrarily close descriptors with lower values. */
-#define SSH_NUM_DUPS		20
-
 /* Force host key length and server key length to differ by at least this
    many bits.  This is to make double encryption with rsaref work. */
 #define SSH_KEY_BITS_RESERVED		128
@@ -197,7 +231,7 @@ void record_login(int pid, const char *ttyname, const char *user, uid_t uid,
 
 /* Records that the user has logged out.  This does many thigs normally
    done by login(1) or init. */
-void record_logout(int pid, const char *ttyname, const char *host);
+void record_logout(int pid, const char *ttyname);
 
 /*------------ definitions for sshconnect.c ----------*/
 
@@ -221,8 +255,8 @@ void ssh_login(RandomState *state, int host_key_valid, RSAPrivateKey *host_key,
 	       unsigned int num_identity_files, char **identity_files,
 	       int rhosts_authentication, int rhosts_rsa_authentication,
 	       int rsa_authentication,
-	       int password_authentication, int cipher_type);
-
+	       int password_authentication, int cipher_type,
+	       const char *system_hostfile, const char *user_hostfile);
 
 /*------------ Definitions for various authentication methods. -------*/
 
@@ -255,6 +289,10 @@ int auth_rsa_read_key(char **cpp, unsigned int *bitsp, MP_INT *e, MP_INT *n);
    cached, so it is efficient to call this several times. */
 const char *get_canonical_hostname();
 
+/* Returns the remote IP address as an ascii string.  The value need not be
+   freed by the caller. */
+const char *get_remote_ipaddr();
+
 /* Tries to match the host name (which must be in all lowercase) against the
    comma-separated sequence of subpatterns (each possibly preceded by ! to 
    indicate negation).  Returns true if there is a positive match; zero
@@ -311,10 +349,27 @@ int load_private_key(const char *filename, const char *passphrase,
 
 /*------------ Definitions for logging. -----------------------*/
 
+/* Supported syslog facilities. */
+typedef enum
+{
+  SYSLOG_FACILITY_DAEMON,
+  SYSLOG_FACILITY_USER,
+  SYSLOG_FACILITY_AUTH,
+  SYSLOG_FACILITY_LOCAL0,
+  SYSLOG_FACILITY_LOCAL1,
+  SYSLOG_FACILITY_LOCAL2,
+  SYSLOG_FACILITY_LOCAL3,
+  SYSLOG_FACILITY_LOCAL4,
+  SYSLOG_FACILITY_LOCAL5,
+  SYSLOG_FACILITY_LOCAL6,
+  SYSLOG_FACILITY_LOCAL7
+} SyslogFacility;
+
 /* Initializes logging.  If debug is non-zero, debug() will output something.
    If quiet is non-zero, none of these will log send anything to syslog
    (but maybe to stderr). */
-void log_init(char *av0, int on_stderr, int debug, int quiet);
+void log_init(char *av0, int on_stderr, int debug, int quiet,
+	      SyslogFacility facility);
 
 /* Outputs a message to syslog or stderr, depending on the implementation. 
    The format must guarantee that the final message does not exceed 1024 
@@ -456,7 +511,7 @@ char *auth_get_socket_name(void);
 
 /* This if called to process SSH_CMSG_AGENT_REQUEST_FORWARDING on the server.
    This starts forwarding authentication requests. */
-void auth_input_request_forwarding(uid_t uid, gid_t gid);
+void auth_input_request_forwarding(struct passwd *pw);
 
 /* This is called to process an SSH_SMSG_AGENT_OPEN message. */
 void auth_input_open_request(void);
@@ -469,4 +524,9 @@ int match_pattern(const char *s, const char *pattern);
    Warning: this calls getpw*. */
 char *tilde_expand_filename(const char *filename, uid_t my_uid);
 
+/* Gets a file descriptor that won't get closed by shell pathname.
+   If pathname is NULL, the path is inferred from the SHELL environment
+   variable or the user id. */
+int get_permanent_fd(const char *pathname);
+
 #endif /* SSH_H */
