@@ -8,7 +8,6 @@ Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
                    All rights reserved
 
 Created: Tue Mar 21 15:59:15 1995 ylo
-Last modified: Fri Jul  7 03:03:49 1995 ylo
 
 Encoding and decoding of terminal modes in a portable way.
 Much of the format is defined in ttymodes.h; it is included multiple times
@@ -17,6 +16,20 @@ suitable code.
 
 */
 
+/*
+ * $Id: ttymodes.c,v 1.3 1995/08/18 22:58:34 ylo Exp $
+ * $Log: ttymodes.c,v $
+ * Revision 1.3  1995/08/18  22:58:34  ylo
+ * 	Added support for NextStep.
+ * 	Fixed typos with EXTB.
+ *
+ * Revision 1.2  1995/07/13  01:41:15  ylo
+ * 	Removed "Last modified" header.
+ * 	Added cvs log.
+ *
+ * $Endlog$
+ */
+
 #include "includes.h"
 #include "packet.h"
 #include "ssh.h"
@@ -32,8 +45,10 @@ suitable code.
 #define cfgetispeed(tio)	((tio)->sg_ispeed)
 #define cfsetospeed(tio, spd)	((tio)->sg_ospeed = (spd), 0)
 #define cfsetispeed(tio, spd)	((tio)->sg_ispeed = (spd), 0)
+#ifndef _NEXT_SOURCE
 typedef char speed_t;
 #endif
+#endif
 
 /* Converts POSIX speed_t to a baud rate.  The values of the constants
    for speed_t are not themselves portable. */
@@ -173,7 +188,7 @@ static speed_t baud_to_speed(int baud)
     case 38400:
       return B38400;
 #else /* B38400 */
-#ifdef /* EXTB */
+#ifdef EXTB
     case 38400:
       return EXTB;
 #endif /* EXTB */
